@@ -25,15 +25,16 @@
 #' generates arrays with nonzero elements in a box, disc or diamond shape for
 #' use with \code{\link{morphology}} functions; \code{gaussianKernel} generates
 #' Gaussian coefficients and is used by \code{\link{gaussianSmooth}};
-#' \code{boxKernel} is used for ``nearest neighbour'' resampling, and
-#' \code{triangleKernel} for linear, bilinear, etc. The Mitchell-Netravali
-#' kernel, a.k.a. BC-spline, is based on a family of piecewise-cubic polynomial
-#' functions, with support of four times the pixel separation in each
-#' dimension. The default parameters are the ones recommended by Mitchell and
-#' Netravali as a good trade-off between various artefacts, but other
-#' well-known special cases include B=1, C=0 (the cubic B-spline) and B=0,
-#' C=0.5 (the Catmull-Rom spline). \code{mnKernel} is a shorter alias for
-#' \code{mitchellNetravaliKernel}.
+#' \code{sobelKernel} generates the Sobel-Feldman gradient operator, for use by
+#' \code{\link{sobelFilter}}; \code{boxKernel} is used for ``nearest
+#' neighbour'' resampling, and \code{triangleKernel} for linear, bilinear, etc.
+#' The Mitchell-Netravali kernel, a.k.a. BC-spline, is based on a family of
+#' piecewise-cubic polynomial functions, with support of four times the pixel
+#' separation in each dimension. The default parameters are the ones
+#' recommended by Mitchell and Netravali as a good trade-off between various
+#' artefacts, but other well-known special cases include B=1, C=0 (the cubic
+#' B-spline) and B=0, C=0.5 (the Catmull-Rom spline). \code{mnKernel} is a
+#' shorter alias for \code{mitchellNetravaliKernel}.
 #' 
 #' @param object Any object.
 #' @param values A numeric vector or array, containing the values of the kernel
@@ -47,7 +48,7 @@
 #'   Does not need to be equal for all dimensions. Will be recycled to length
 #'   \code{dim} if that parameter is also specified.
 #' @param dim An integer value giving the dimensionality of the kernel.
-#'   Defaults to the length of \code{width} or \code{sigma}.
+#'   Defaults to the length of \code{width} or \code{sigma}, where available.
 #' @param size A numeric vector giving the width of the kernel in each
 #'   dimension, which will be rounded up to the nearest odd integer. Defaults
 #'   to four times the corresponding \code{sigma} value.
@@ -60,14 +61,15 @@
 #' @param normalised If \code{TRUE}, the sum of non-missing elements of the
 #'   kernel will be unity. Note that this is the default for
 #'   \code{gaussianKernel}, but not for \code{shapeKernel}.
+#' @param axis The axis along which the gradient operator will be applied.
 #' @param name A string giving the name of the kernel function required.
 #' @param \dots Parameters for the kernel function.
 #' @param B,C Mitchell-Netravali coefficients, each of which must be between 0
 #'   and 1.
 #' @return For \code{isKernel}, \code{isKernelArray} and
 #'   \code{isKernelFunction}, a logical value. For \code{kernelArray},
-#'   \code{shapeKernel} and \code{gaussianKernel}, a kernel array. For
-#'   \code{kernelFunction}, \code{boxKernel}, \code{triangleKernel},
+#'   \code{shapeKernel}, \code{gaussianKernel} and \code{sobelKernel}, a kernel
+#'   array. For \code{kernelFunction}, \code{boxKernel}, \code{triangleKernel},
 #'   \code{mitchellNetravaliKernel} and \code{mnKernel}, a kernel function.
 #' 
 #' @examples
@@ -248,7 +250,7 @@ gaussianKernel <- function (sigma, dim = length(sigma), size = 6*sigma, normalis
     
     size <- ifelse(size %% 2 == 1, size, size+1)
     
-    scaleFactors <- max(sigma) / sigma
+    scaleFactors <- max(sigma) / ifelse(sigma==0, 1, sigma)
     x <- lapply(1:dim, function(i) 1:size[i] - (size[i]+1)/2)
     centres <- lapply(1:dim, function(i) scaleFactors[i] * x[[i]])
     
@@ -263,6 +265,21 @@ gaussianKernel <- function (sigma, dim = length(sigma), size = 6*sigma, normalis
     return (kernelArray(kernel))
 }
 
+#' @rdname kernels
+#' @export
+sobelKernel <- function (dim, axis = 1)
+{
+    if (!(axis %in% 0:dim))
+        report(OL$Error, "The axis should be between 0 and #{dim}")
+    
+    parts <- rep(list(c(1,2,1) / 4), dim)
+    if (axis > 0)
+        parts[[axis]] <- c(1, 0, -1)
+    
+    kernel <- Reduce(outer, parts)
+    return (kernelArray(kernel))
+}
+
 #' @rdname kernels
 #' @export
 kernelFunction <- function (name = c("box","triangle","mitchell-netravali"), ...)
